CREATE OR REPLACE PACKAGE BODY trainingmgr.xx_plan_app
------------------------------------------------------------------
--
-- $Revision:   1.0  $
-- $Header:   //norton_samba/pvcsdb/pvcsdata/pvcsdb01/archives/Training/Training Example Code/XX_PLANNING_APPLICATION.pkb-arc   1.0   Jul 12 2012 14:56:22   tarthey  $
--
------------------------------------------------------------------
IS

  -- Creates a new application type
  FUNCTION create_application_type (
    p_title xx_plan_app_types.title%TYPE
  , p_price xx_plan_app_types.price%TYPE
  , p_description xx_plan_app_types.description%TYPE
  )
  RETURN xx_plan_app_types.id%TYPE
  IS
    
    l_application_type_id xx_plan_app_types.id%TYPE;
    
  BEGIN
  
    SELECT xx_plan_app_types_seq.nextVal
    INTO l_application_type_id 
    FROM dual;
  
    INSERT INTO xx_plan_app_types (
      id
    , start_datetime
    , end_datetime
    , title
    , price
    , description
    )
    VALUES (
      l_application_type_id
    , SYSDATE
    , NULL
    , p_title
    , p_price
    , p_description
    );
    
    RETURN l_application_type_id;
    
  END create_application_type;
  
  -- Create planning application reference from application id.
  FUNCTION create_application_reference (
    p_application_id xx_plan_apps.id%TYPE
  ) 
  RETURN xx_plan_apps.application_reference%TYPE
  IS
  BEGIN
    RETURN 'PLAN/' || TO_CHAR(SYSDATE, 'YYYY/MM/') || p_application_id;
  END create_application_reference;

  -- Creates a new draft application  
  FUNCTION create_new_application (
    p_application_type_id xx_plan_app_types.id%TYPE
  ) 
  RETURN xx_plan_apps.id%TYPE
  IS
    l_application_id xx_plan_apps.id%TYPE;
    l_application_detail_id xx_plan_app_details.id%TYPE;
    
  BEGIN
    SELECT
      xx_plan_apps_seq.nextval 
    , xx_plan_app_details_seq.nextval
    INTO
      l_application_id 
    , l_application_detail_id
    FROM dual;
        
    INSERT INTO xx_plan_apps (
      id
    , type_id
    , application_reference
    )
    VALUES (
      l_application_id
    , p_application_type_id
    , create_application_reference(l_application_id)
    );
    
    INSERT INTO xx_plan_app_details (
      id
    , application_id
    , start_datetime
    , status_control
    , status
    , xml_data
    )
    VALUES (
      l_application_detail_id
    , l_application_id
    , SYSDATE
    , 'C'
    , 'DRAFT'
    , XMLELEMENT("APPLICATION_DETAIL")
    );

    RETURN l_application_id;
  END create_new_application;
  
  -- Updates an existing application data.
  FUNCTION update_application_data (
    p_application_id xx_plan_apps.id%TYPE
  , p_xml_data xx_plan_app_details.xml_data%TYPE
  )
  RETURN xx_plan_app_details.id%TYPE
  IS
    l_status xx_plan_app_details.status%TYPE;
    l_application_detail_id xx_plan_app_details.id%TYPE;
      
  BEGIN 
    UPDATE xx_plan_app_details 
    SET 
      status_control = NULL
    , end_datetime = SYSDATE
    WHERE 
        status_control = 'C'
    AND end_datetime IS NULL
    AND application_id = p_application_id
    RETURNING status
    INTO l_status;
    
    SELECT xx_plan_app_details_seq.nextval
    INTO l_application_detail_id
    FROM dual;

    INSERT INTO xx_plan_app_details (
      id
    , application_id
    , start_datetime
    , status_control
    , status
    , xml_data
    )
    VALUES (
      l_application_detail_id
    , p_application_id
    , SYSDATE
    , 'C'
    , l_status
    , p_xml_data
    );
    
    RETURN l_application_detail_id;    
  END;
END xx_plan_app;
/
